package com.yixin.ms.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.UUID;
import cn.hutool.core.util.DesensitizedUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.excel.util.CollectionUtils;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.beust.jcommander.internal.Lists;
import com.yixin.ms.boot.restful.RestResponse;
import com.yixin.ms.boot.restful.RestResult;
import com.yixin.ms.boot.restful.ServiceException;
import com.yixin.ms.boot.uitls.DESUtil;
import com.yixin.ms.boot.uitls.OkHttpUtils;
import com.yixin.ms.boot.uitls.RedisUtil;
import com.yixin.ms.model.dto.HealthCodeDTO;
import com.yixin.ms.model.dto.ShortTermDTO;
import com.yixin.ms.model.enums.*;
import com.yixin.ms.model.vo.HealtCodeNucleinVO;
import com.yixin.ms.model.vo.HealtCodeTypeVO;
import com.yixin.ms.model.vo.HealtCodeUserVO;
import com.yixin.ms.server.HealthCodeServer;
import com.yixin.ms.server.JiliangUniversityServer;
import com.yixin.ms.server.dto.JiliangCanPassDataDTO;
import com.yixin.ms.server.vo.HealthCodeVO;
import com.yixin.ms.service.HealthCodeService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Executor;


@Slf4j
@Service
public class HealthCodeServiceImpl implements HealthCodeService {
    @Autowired
    private RedisUtil redisUtil;
    @Autowired
    private HealthCodeServer healthCodeServer;
    @Autowired
    private JiliangUniversityServer jiliangUniversityServer;
    @Autowired
    private Executor executor;

    @Override
    public HealtCodeUserVO getShortTermCertificate(ShortTermDTO shortTermDTO) {
        try {
            HealtCodeUserVO shortTermUser = getShortTermUser(shortTermDTO.getIdNumber(), shortTermDTO.getName());
            if (shortTermUser != null && !CollectionUtils.isEmpty(shortTermUser.getData())) {
                return shortTermUser;
            }
            return getSjJkm(shortTermDTO.getIdNumber(), shortTermDTO.getName());
        } catch (Exception e) {
            e.printStackTrace();
            log.info("异常 :" + e.getMessage());
            try {
                return getSjJkm(shortTermDTO.getIdNumber(), shortTermDTO.getName());
            } catch (Exception ex) {
                ex.printStackTrace();
                throw new ServiceException("未获取到健康码信息，请联系工作人员");
            }
        }

    }

    @Override
    public String addToken(String token) {
        String s = "hh_token_" + UUID.randomUUID();
        boolean hanghui = redisUtil.set(s, s);
        if (hanghui) {
            return s;
        } else {
            throw new ServiceException("体检失败");
        }

    }

    @Override
    public Boolean deleteToken(String token) {
        return redisUtil.delete(token);
    }

    @Override
    public RestResult getShortTerm(ShortTermDTO shortTermDTO) {
        log.info("健康码查询入参信息：{}", JSON.toJSONString(shortTermDTO));
        if (StringUtils.isBlank(shortTermDTO.getToken())) {
            throw new ServiceException(700, "token为空哦");
        }

        if (StrUtil.isEmpty(shortTermDTO.getHealthCode())) {
            if (StringUtils.isBlank(shortTermDTO.getIdNumber()) && StringUtils.isBlank(shortTermDTO.getName())) {
                throw new ServiceException(701, "身份证和姓名为空哦");
            }
        }

        String canPass = jiliangUniversityServer.canPass(new JiliangCanPassDataDTO().setIdCard(shortTermDTO.getIdNumber()).setOpTime(DateUtil.date()).setSn(shortTermDTO.getSn()));
        if(canPass.equals(CanPassEnum.NOT_THROUGH.getCode())){
            return RestResponse.error("暂无通行权限，请联系工作人员");
        }

        /**
         * 核酸有效期时间限制
         */
        Integer nucleicExpirationTime = 0;
        //设备sn为空默认走之前的健康码查询
        HealtCodeUserVO userVO = this.getShortTermCertificate(shortTermDTO);

        if (userVO != null) {
            if (userVO.getData() == null || userVO.getData().isEmpty()) {
                throw new ServiceException("未查询到健康吗");
            }

            //姓名
            userVO.setUsername(StrUtil.isEmpty(userVO.getUsername()) ? shortTermDTO.getName() : userVO.getUsername());
            //身份证号
            userVO.setIdNumber(StrUtil.isEmpty(userVO.getIdNumber()) ? shortTermDTO.getIdNumber() : userVO.getIdNumber());
            //姓名和身份证号脱敏处理
            userVO.setUsername(StrUtil.isEmpty(userVO.getUsername()) ? StrUtil.EMPTY : DesensitizedUtil.chineseName(userVO.getUsername()));
            userVO.setIdNumber(StrUtil.isEmpty(userVO.getIdNumber()) ? StrUtil.EMPTY : DesensitizedUtil.idCardNum(userVO.getIdNumber(), 4, 4));
            /**
             * 0 未接种 对应0针
             *
             * 1-接种完成 对应2针
             *
             * 2-未完全接种 对应1针
             *
             * 3- 完成加强针 对应3针
             */
            if (HealthVaccineNumEnum.VACCINATION_COMPLETE.getStatus().equals(userVO.getVaccineNum())) {
                userVO.setVaccineNum(HealthVaccineNumEnum.VACCINATION_COMPLETE.getNum());
            } else if (HealthVaccineNumEnum.INCOMPLETE_INOCULATION.getStatus().equals(userVO.getVaccineNum())) {
                userVO.setVaccineNum(HealthVaccineNumEnum.INCOMPLETE_INOCULATION.getNum());
            } else {
                userVO.setVaccineNum(userVO.getVaccineNum());
            }

            //用户健康码信息
            HealtCodeTypeVO codeTypeVO = userVO.getData().stream().findFirst().get();
            //用户核酸信息
            List<HealtCodeNucleinVO> nucleinList = userVO.getNuclein();
            if (CollectionUtil.isNotEmpty(nucleinList)) {
                nucleinList.forEach(v -> v.setShowTime(v.getReportTime() == null ? v.getChecktime() : DateUtil.compare(v.getChecktime(), v.getReportTime()) > 0 ? v.getChecktime() : v.getReportTime()));
            }

            if (HealthCodeLevelEnum.GREEN.getDesc().equals(codeTypeVO.getMzt())) {
                codeTypeVO.setContent("绿码：健康码状态为低风险").setContentColor(ColorEnum.GREEN.getValue());
                userVO.setBackgroundColor(ColorEnum.GREEN.getValue());
            }

            //核酸过期标识
            boolean nucleicExpirationFlag = false;
            /**
             * 开门规则判定校验：红黄码，核酸有效期
             */
            if (!HealthCodeLevelEnum.GREEN.getDesc().equals(codeTypeVO.getMzt())) {
                userVO.setIsOpen(HealthOpenEnum.NO_THOROUGHFARE.getCode());
                String openMsg = new StringBuilder().append(codeTypeVO.getMzt()).append("禁止通行").toString();
                userVO.setOpenMsg(openMsg).setOpenVoiceMsg(openMsg);
                if (HealthCodeLevelEnum.RED.getDesc().equals(codeTypeVO.getMzt())) {
                    codeTypeVO.setContent(new StringBuilder().append(codeTypeVO.getMzt()).append("：健康码状态为高风险").toString()).setContentColor(ColorEnum.RED.getValue());
                    userVO.setBackgroundColor(ColorEnum.RED.getValue());
                } else {
                    codeTypeVO.setContent(new StringBuilder().append(codeTypeVO.getMzt()).append(": 健康码状态为中风险").toString()).setContentColor(ColorEnum.YELLOW.getValue());
                    userVO.setBackgroundColor(ColorEnum.YELLOW.getValue());
                }
            } else if (nucleicExpirationTime != null && !nucleicExpirationTime.equals(0) && (CollectionUtils.isEmpty(nucleinList) || DateUtil.betweenMs(nucleinList.stream().findFirst().get().getShowTime(), DateUtil.date()) > nucleicExpirationTime * 60 * 60 * 1000)) {
                nucleicExpirationFlag = true;
                userVO.setIsOpen(HealthOpenEnum.NO_THOROUGHFARE.getCode());
                String openMsg;
                if (nucleicExpirationTime <= NucleicRuleEnum.SEVENTY_TWO.getValue()) {
                    openMsg = new StringBuilder().append(nucleicExpirationTime).append("小时内无有效核酸阴性记录，禁止通行").toString();
                } else {
                    openMsg = new StringBuilder().append((nucleicExpirationTime / 24) + 1).append("天内无有效核酸阴性记录，禁止通行").toString();
                }
                userVO.setOpenMsg(openMsg).setOpenVoiceMsg(openMsg);
            } else {
                userVO.setIsOpen(HealthOpenEnum.RELEASE.getCode()).setOpenMsg("绿码，请通行").setOpenVoiceMsg("绿码，请通行");
            }

            if (CollectionUtil.isNotEmpty(nucleinList)) {
                HealtCodeNucleinVO nucleinVO = nucleinList.stream().findFirst().get();
                nucleinVO.setTitle("核酸检测").setContentColor(nucleicExpirationFlag ? ColorEnum.YELLOW.getValue() : ColorEnum.GREEN.getValue()).setIconUrl(nucleicExpirationFlag ? NucleicStatusEnum.NOT_AVAILABLE.getUrl() : NucleicStatusEnum.NEGATIVE.getUrl());
                if (nucleinVO.getReportTime() != null && DateUtil.compare(nucleinVO.getChecktime(), nucleinVO.getReportTime()) > 0) {
                    nucleinVO.setContent(new StringBuilder().append("结果未出").toString());
                } else {
                    long betweenHour = DateUtil.between(nucleinVO.getShowTime(), DateUtil.date(), DateUnit.HOUR);
                    if (betweenHour < NucleicRuleEnum.TWENTY_FOUR.getValue()) {
                        nucleinVO.setContent(new StringBuilder().append(NucleicRuleEnum.TWENTY_FOUR.getValue()).append(" ").append(nucleinVO.getResult()).toString());
                    } else if (betweenHour < NucleicRuleEnum.FORTY_EIGHT.getValue()) {
                        nucleinVO.setContent(new StringBuilder().append(NucleicRuleEnum.FORTY_EIGHT.getValue()).append(" ").append(nucleinVO.getResult()).toString());
                    } else if (betweenHour < NucleicRuleEnum.SEVENTY_TWO.getValue()) {
                        nucleinVO.setContent(new StringBuilder().append(NucleicRuleEnum.SEVENTY_TWO.getValue()).append(" ").append(nucleinVO.getResult()).toString());
                    } else {
                        nucleinVO.setContent(new StringBuilder().append(DateUtil.betweenDay(nucleinVO.getShowTime(), DateUtil.date(), false) + 1).append("天 ").append(nucleinVO.getResult()).toString());
                    }
                }
            } else {
                userVO.setNuclein(Lists.newArrayList(new HealtCodeNucleinVO().setTitle("核酸检测").setIconUrl(NucleicStatusEnum.NOT_AVAILABLE.getUrl()).setContent("暂无").setContentColor(ColorEnum.GREEN.getValue())));
            }

            userVO.setTitle("健 康 码").setTitleColor(ColorEnum.WHITE.getValue())
                    //疫苗信息
                    .setVaccineIcoTitle("新冠疫苗").setVaccineIconUrl(HealthVaccineNumEnum.numOf(userVO.getVaccineNum()).getUrl()).setVaccineNumColor(ColorEnum.GREEN.getValue())
                    //行程信息
                    .setTravelValidationDesc(TravelValidationEnum.codeOf(userVO.getTravelValidation()).getDesc()).setTravelValidationTitle("动态行程卡").setTravelValidationIconUrl(TravelValidationEnum.codeOf(userVO.getTravelValidation()).getUrl());
            return RestResponse.ok(userVO);
        }

        throw new ServiceException(703, "未查询到此数据");
    }

    @Override
    public RestResult<HealthCodeVO> queryInfoByBarCode(HealthCodeDTO healthCodeDTO) {
        IrsTripAuthEnum authEnum = IrsTripAuthEnum.appKeyOf(healthCodeDTO.getAppKey());
        if (authEnum == null || !authEnum.getAppSecret().equals(healthCodeDTO.getAppSecret())) {
            return RestResponse.error("调用权限校验失败，请检查");
        }

        return healthCodeServer.queryInfoByBarCode(healthCodeDTO.getBarCode());
    }

    public HealtCodeUserVO getShortTermUser(String sfzh, String name) throws Exception {
        log.info("江干分局健康码传参，身份证{} 姓名 {}", sfzh, name);
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("sfzh", sfzh);
        jsonObject.put("xm", name);
        String sync = OkHttpUtils.builder().url("http://115.236.38.205:8903/id/hvn")
                .addHeader("Content-Type", "application/json; charset=utf-8")
                .localPost(jsonObject.toJSONString())
                .sync();
        if (sync.contains("请求失败：Connect timed out")) {
            throw new ServiceException("江干分局请重试");
        }
        log.info("江干分局健康码返回结果，{}", sync);
        HealtCodeUserVO healtCodeUserVO = new HealtCodeUserVO();
        if (StringUtils.isNotBlank(sync)) {
            JSONObject json = JSON.parseObject(sync);
            if (StringUtils.equals("00", json.getString("error_code"))) {
                healtCodeUserVO.setData(JSON.parseArray(json.getJSONArray("data").toJSONString(), HealtCodeTypeVO.class));
                healtCodeUserVO.setNuclein(JSON.parseArray(json.getJSONArray("nuclein").toJSONString(), HealtCodeNucleinVO.class));
                //源数据只返回报告时间，检测时间先等同于报告时间，方便后面逻辑处理
//                healtCodeUserVO.getNuclein().forEach(v -> v.setReportTime(v.getChecktime()));
                healtCodeUserVO.setVaccineNum(json.getString("vaccine_num"));
            } else {
                throw new ServiceException("江干分局异常:" + jsonObject.getString("msg"));
            }
        }
        return healtCodeUserVO;
    }

    public HealtCodeUserVO getSjJkm(String sfzh, String name) throws Exception {
        log.info("市局健康码传参，身份证{} 姓名 {}", sfzh, name);
        String token = "";
        token = (String) redisUtil.get("jkm-token");
        if (StringUtils.isBlank(token)) {
            token = getToken();
            if (StringUtils.isBlank(token)) {
                throw new ServiceException("市局健康码请重试");
            }
        }
        JSONObject data = new JSONObject();
        JSONObject body = new JSONObject();
        body.put("idCard", sfzh);
        String encryptSfzh = DESUtil.encrypt(body.toJSONString(), "7760ce65a8544f779f36607c201e8a0c");
        data.put("body", encryptSfzh);
        data.put("token", token);
        String sync = OkHttpUtils.builder().url("http://policecard.tenzont.com/auth/api/healthCode")
                .addHeader("Content-Type", "application/json; charset=utf-8")
                .localPost(data.toJSONString())
                .sync();
        if (sync.contains("请求失败：Connect timed out")) {
            throw new ServiceException("市局健康码请重试");
        }
        log.info("市局健康码返回结果，{}", sync);
        JSONObject jkm = JSONObject.parseObject(sync);
        String rspCode = jkm.getJSONObject("header").getString("rspCode");
        HealtCodeUserVO healtCodeUserVO = new HealtCodeUserVO();
        if (StringUtils.equals("00", rspCode)) {
            String mzt = jkm.getJSONObject("body").getString("mzt");
            if (StringUtils.isNotBlank(mzt)) {
                List<HealtCodeTypeVO> list = new ArrayList<>();
                HealtCodeTypeVO healtCodeTypeVO = new HealtCodeTypeVO();
                healtCodeTypeVO.setMzt(mzt);
                list.add(healtCodeTypeVO);
                healtCodeUserVO.setData(list);
                healtCodeUserVO.setNuclein(new ArrayList<>());
                healtCodeUserVO.setVaccineNum("");
                return healtCodeUserVO;
            } else {
                throw new ServiceException("市局健康码请重试");
            }
        } else if (StringUtils.equals("01", rspCode)) {
            getToken();
            HealtCodeUserVO sjJkm = getSjJkm(sfzh, name);
            return sjJkm;
        } else {
            throw new ServiceException("市局健康码请重试");
        }
    }


    public String getToken() throws Exception {
        String sync = "";
        JSONObject jsonObject = new JSONObject();
        JSONObject body = new JSONObject();
        body.put("appid", "jzz3c086d786420a958a143e6154a351");
        body.put("secret", "ecbe0e220ec74d05b2b22268a737d2f1");
        jsonObject.put("body", body);
        try {
            sync = OkHttpUtils
                    .builder()
                    .url("http://policecard.tenzont.com/auth/api/token")
                    .localPost(jsonObject.toJSONString())
                    .sync();
        } catch (Exception e) {
            log.info("获取健康码的token接口异常 ，{}", e.getMessage());
            return "";
        }
        if (StringUtils.isBlank(sync)) {
            return "";
        }
        JSONObject fhz = JSONObject.parseObject(sync);
        String rspCode = fhz.getJSONObject("header").getString("rspCode");
        if (StringUtils.equals("00", rspCode)) {
            String token = fhz.getJSONObject("body").getString("token");
            if (StringUtils.isNotBlank(token)) {
                redisUtil.set("jkm-token", token, 7200);
                return token;
            }
        }
        return "";
    }

//    public static void main(String[] args) throws Exception {
//        System.out.println(getSjJkm("412728199706030513", ""));
//    }
}

